//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * ベクトル配列補間ヘッダ
 * @author Junpee
 */

#ifndef VECTOR_ARRAY_INTERPOLATOR_H_
#define VECTOR_ARRAY_INTERPOLATOR_H_

#include <Animation/VectorInterpolator/VectorInterpolator.h>

namespace Lamp{

//------------------------------------------------------------------------------
/**
 * ベクトル配列補間
 */
class VectorArrayInterpolator : public VectorInterpolator{
public:
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 */
	VectorArrayInterpolator();

	/**
	 * デストラクタ
	 */
	virtual ~VectorArrayInterpolator();

	/**
	 * コピーコンストラクタ
	 * @param copy コピー元
	 */
	explicit VectorArrayInterpolator(const VectorArrayInterpolator& copy);

	/**
	 * 代入演算子
	 * @param copy 代入元
	 */
	virtual VectorArrayInterpolator& operator =(
		const VectorArrayInterpolator& copy);

	//--------------------------------------------------------------------------
	/**
	 * 複製
	 * @return 複製されたベクトル補間。呼び出し元でdeleteする必要がある
	 */
	virtual VectorInterpolator* duplicate() const{
		VectorInterpolator* result = new VectorArrayInterpolator(*this);
		return result;
	}

	//--------------------------------------------------------------------------
	/**
	 * 同じ値かどうか
	 * @param target 比較対象
	 * @return 同じ値ならtrueをかえす
	 */
	virtual bool equals(const VectorInterpolator& target) const{
		VectorArrayInterpolator* interpolator =
			target.castVectorArrayInterpolator();
		if(interpolator == NULL){ return false; }
		if(size_ != interpolator->size_){ return false; }
		for(int i = 0; i < size_; i++){
			if(array_[i] != interpolator->array_[i]){ return false; }
		}
		return true;
	}

	//--------------------------------------------------------------------------
	// 長さ
	//--------------------------------------------------------------------------
	/**
	 * 長さの取得
	 * @return 長さ
	 */
	virtual float getLength() const{ return length_; }

	//--------------------------------------------------------------------------
	// バウンディング
	//--------------------------------------------------------------------------
	/**
	 * バウンディングボックスの取得
	 * @return バウンディングボックス
	 */
	virtual AxisAlignedBox getBoundingBox() const;

	//--------------------------------------------------------------------------
	// 補間
	//--------------------------------------------------------------------------
	/**
	 * 補間
	 * @param time 時間
	 * @return 補間されたベクトル
	 */
	virtual Vector3 interpolate(float time);

	//--------------------------------------------------------------------------
	// 値
	//--------------------------------------------------------------------------
	/**
	 * サイズ設定
	 * @param size 設定するサイズ。サイズ - 1がLengthになる。
	 */
	virtual void setSize(int size);

	/**
	 * サイズ取得
	 * @return サイズ
	 */
	virtual int getSize() const{ return size_; }

	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param time 値を設定する時間
	 * @param value 設定する値
	 */
	virtual void setValue(int time, const Vector3& value){
		Assert((array_ != NULL) && (time >= 0) && (time < size_));
		array_[time] = value;
	}

	/**
	 * 値の取得
	 * @param time 値を取得する時間
	 * @return 値
	 */
	virtual const Vector3& getValue(int time) const{
		Assert((array_ != NULL) && (time >= 0) && (time < size_));
		return array_[time];
	}

	//--------------------------------------------------------------------------
	// RTTI
	//--------------------------------------------------------------------------
	/**
	 * ベクトル配列補間かどうか
	 * @return ベクトル配列補間ならtrue
	 */
	virtual bool isVectorArrayInterpolator() const{ return true; }

	//--------------------------------------------------------------------------
private:
	// 配列
	Vector3* array_;
	// サイズ
	int size_;
	// 長さ
	float length_;

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of VECTOR_ARRAY_INTERPOLATOR_H_
//------------------------------------------------------------------------------

